$expected = 0;
#$fname = "gray150.txt";

# C functions
print <<cut
#include <stdio.h>
#include <usb.h>
#include <stdarg.h>

int rd_timeout = 10000;
int wr_timeout = 10000;
unsigned char rd_byte;

usb_dev_handle *find_device(unsigned int idVendor, unsigned int idProduct)
{
  struct usb_bus *bus;
  struct usb_device *dev;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus; bus = bus->next)
    for (dev = bus->devices; dev; dev = dev->next)
      if (dev->descriptor.idVendor==idVendor && dev->descriptor.idProduct==idProduct)
	return usb_open(dev);

  return NULL;
}

/* CW: 40 0c 0090 0000 0001 :  64 */
/* CW: 40 0c 0090 0000 0001 :  65 */
/* CW: 40 0c 0090 0000 0001 :  44 */
/* CW: 40 0c 0090 0000 0001 :  45 */
void idle_b(usb_dev_handle *udev)
{
    int len, i;
    unsigned char buff[4] = {0x64, 0x65, 0x44, 0x45};
    for (i=0; i<4; i++) {
	len = usb_control_msg(udev, 0x40, 0x0c, 0x0090, 0x0000, buff+i, 0x0001, wr_timeout);
    }
}

/* CW: 40 0c 0090 0000 0001 :  64 */
/* CW: 40 0c 0090 0000 0001 :  65 */
/* CW: 40 0c 0090 0000 0001 :  16 */
/* CW: 40 0c 0090 0000 0001 :  17 */
void idle_a(usb_dev_handle *udev)
{
    int len, i;
    unsigned char buff[4] = {0x64, 0x65, 0x16, 0x17};
    for (i=0; i<4; i++) {
	len = usb_control_msg(udev, 0x40, 0x0c, 0x0090, 0x0000, buff+i, 0x0001, wr_timeout);
    }
}

void idle_ab(usb_dev_handle *udev)
{
    idle_a(udev);
    idle_b(udev);
}

/* CW: 40 04 00b0 0000 <len> :  <reg1> <value1> <reg2> <value2> ... */
void write_regs(usb_dev_handle *udev, int regs, unsigned char reg1, unsigned char val1, ... /*unsigned char reg, unsigned char val, ...*/ )
{
    unsigned char buff[512];
    va_list marker;
    int len, i;

    va_start(marker, val1);
    buff[0] = reg1;
    buff[1] = val1;
    for (i=1; i<regs; i++) {
	buff[i*2] = va_arg(marker, int);
	buff[i*2+1] = va_arg(marker, int);
    }
    va_end(marker);

    len = usb_control_msg(udev, 0x40, 0x04, 0x00b0, 0x0000, buff, regs*2, wr_timeout);
}

int write_vctl(usb_dev_handle *udev, int request, int value, int index, unsigned char byte)
{
    return usb_control_msg(udev, 0x40, request, value, index, &byte, 1, wr_timeout);
}

int read_vctl(usb_dev_handle *udev, int request, int value, int index, unsigned char *byte)
{
    return usb_control_msg(udev, 0xc0, request, value, index, byte, 1, wr_timeout);
}

void test(usb_dev_handle *udev);

int main()
{
    test(find_device(0x05da, 0x30d4));
    return 0;
}


void test(usb_dev_handle *udev)
{
    int len;

    len = usb_set_configuration(udev, 1);
    len = usb_claim_interface(udev, 0);
    len = usb_clear_halt(udev, 1);
    len = usb_clear_halt(udev, 2);
    len = usb_clear_halt(udev, 3);
    /* } */
cut
;

die "need fname\n" if (!@ARGV);
($fname) = @ARGV;
open F, "<$fname";
while ($l=<F>) {
    $pos = tell(F);
    chomp $l;
    $a = $l;
    $a =~ s/([0-9]+) //;
    $urb = $1;
#		print "//$urb\n";
    if ( ($a eq "CW: 40 0c 0090 0000 0001 :  64") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  65") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  16") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  17") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  64") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  65") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  44") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  45") ) {
				print "\tidle_ab(udev);\n";
				next;
    }
    seek(F, $pos, SEEK_SET);
    if ( ($a eq "CW: 40 0c 0090 0000 0001 :  64") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  65") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  16") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  17") ) {
				print "\tidle_a(udev);\n";
				next;
    }
    seek(F, $pos, SEEK_SET);
    if ( ($a eq "CW: 40 0c 0090 0000 0001 :  64") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  65") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  44") &&
				 (&getline eq "CW: 40 0c 0090 0000 0001 :  45") ) {
				print "\tidle_b(udev);\n";
				next;
    }
    seek(F, $pos, SEEK_SET);
    
    if ($a =~ /CW\: 40 04 00b0 0000 /) {
				$a =~ s/CW\: 40 04 00b0 0000 //;
				($len) = split(/\s+/, $a);
				$len = hex($len) / 2;
				$a =~ s/[0-9a-f]+ \: //;
				$a =~ s/ /\, 0x/g;
				print "\twrite_regs(udev, $len$a);\n";
				next;
    }
    
    if ($a =~ /CW\: (40) ([0-9a-f]+) ([0-9a-f]+) ([0-9a-f]+) 0001 \:  ([0-9a-f]+)$/) {
				print "\twrite_vctl(udev, 0x$2, 0x$3, 0x$4, 0x$5);\n";
				next;
    }
    
    if ($a =~ /CR\: (c0) ([0-9a-f]+) ([0-9a-f]+) ([0-9a-f]+) 0001 \:  ([0-9a-f]+)$/) {
				print "\tread_vctl(udev, 0x$2, 0x$3, 0x$4, &rd_byte);  /* expect: 0x$5 */ \n";
				next;
    }
		
		
    if ($a =~ /CR\: (80) ([0-9a-f]+) ([0-9a-f]+) ([0-9a-f]+) ([0-9a-f]+) \:  /) {
				($bmRequestType, $bRequest, $wValue, $wIndex, $wLength) = ($1, $2, $3, $4, $5);
				$a =~ s/.*\:\s+//;
				$a =~ s/\s+/\,0x/g;
				print "\tunsigned char exp_$urb"."[0x$wLength]={0x$a};\n" if ($expected);
				print "\tunsigned char msg_$urb"."[0x$wLength];\n";
				print "\tlen = usb_control_msg(udev, 0x$bmRequestType, 0x$bRequest, 0x$wValue, 0x$wIndex, msg_$urb, 0x$wLength, rd_timeout);\n";
				next;
    }
		
		
    if ($a =~ /BW ep\(([0-9]+)\) \[([0-9]+)\]\:  /) {
				$ep = $1;
				$count = $2;
				$a =~ s/.*\:\s+//;
				$a =~ s/\s+/\,0x/g;
				# chop off trailing zeros
				$a =~ s/(,0x00)+,0x00$/,0x00/g;
				$a =~ s/0x00/0/g;
				print "\tunsigned char msg_$urb"."[$count] = {0x$a};\n";
				print "\tlen = usb_bulk_write(udev, $ep, msg_$urb, $count, wr_timeout);\n";
				next;
    }
		
		
    if ($a =~ /BR ep\(([0-9]+)\) \[([0-9]+)\]\:  /) {
				$ep = $1;
				$count = $2;
				$a =~ s/.*\:\s+//;
				$a =~ s/\s+/\,0x/g;
				# combine multiple reads
				do {
						$l = &getline;
						$ok = 0;
						if ($l =~ /BR ep\(([0-9]+)\) \[([0-9]+)\]\:  /) {
								$nep = $1;
								$ncount = $2;
								$l =~ s/.*\:\s+//;
								$l =~ s/\s+/\,0x/g;
								if ($nep==$ep) {
										$ok = 1;
										$count = $count + $ncount;
										$a = $a . ",0x" . $l if ($expected);
										$pos = tell(F);
								}
						}
				} while ($ok==1);
				seek(F, $pos, SEEK_SET);
				print "\tunsigned char exp_$urb"."[$count] = {0x$a};\n" if ($expected);
				if ($count==1) {
						print "\tlen = usb_bulk_read(udev, $ep, &rd_byte, 1, rd_timeout);\n";
				} else {
						printf "\trecord(udev, \"record.%03d\", $count);\n", $reccnt++;
#						print "\tunsigned char msg_$urb"."[$count];\n";
#						print "\tlen = usb_bulk_read(udev, $ep, msg_$urb, $count, rd_timeout);\n";
				}
				next;
    }
		
		
		
    print "$a\n";
    
}
close F;
print "\n}\n\n";



sub getline 
{
    my $a=<F>;
    chomp $a;
    $a =~ s/[0-9]+ //;
    return $a;
}
